#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <libev/ev.h>


#define CHILD_COUNT 5


ev_child childs[CHILD_COUNT];


static void start_child(ev_child *child);

static void exit_on_signal(EV_P_ ev_signal *w, int revents)
{
    int i;

    printf("Watchdog take %d signal\n", w->signum);

    for (i = 0; i < CHILD_COUNT; i++) {
	ev_child_stop(EV_A_ &childs[i]);
	kill(childs[i].pid, SIGTERM);
    }

    ev_unloop(EV_A_ EVUNLOOP_ALL);
}

static void child_exit_on_signal(EV_P_ ev_signal *w, int revents)
{
    printf("Child take %d signal\n", w->signum);
    ev_unloop(EV_A_ EVUNLOOP_ALL);
}

static void child_exit(EV_P_ ev_child *w, int revents)
{
    ev_child_stop (EV_A_ w);
    printf("process %d exited with status %x\n", w->rpid, w->rstatus);
    start_child(w);
}

static void start_child(ev_child *child)
{
    pid_t pid;

    pid = fork();
    if (pid == -1) {
	perror("fork() failed");
    }

    /* child */    
    if (pid == 0) {
	ev_signal       sigint;
	ev_signal       sighup;
	ev_signal       sigterm;

	ev_signal_init(&sigint, child_exit_on_signal, SIGINT);
	ev_signal_init(&sighup, child_exit_on_signal, SIGHUP);
	ev_signal_init(&sigterm, child_exit_on_signal, SIGTERM);

	ev_signal_start(EV_DEFAULT_ &sigint);
	ev_signal_start(EV_DEFAULT_ &sighup);
	ev_signal_start(EV_DEFAULT_ &sigterm);

	ev_loop(EV_DEFAULT_ 0);

	exit(0);
    }
    /* parent */    
    printf("setup the child watcher for pid %d\n", pid);

    /* fresh EV_DEFAULT */
    ev_loop_destroy(EV_DEFAULT);
    ev_default_loop(0);

    ev_child_init(child, child_exit, pid, 0);
    ev_child_start(EV_DEFAULT_ child);
}


int main(int args, char *argv[]) {
    int i;

    ev_signal       sigint;
    ev_signal       sighup;
    ev_signal       sigterm;

    for (i = 0; i < CHILD_COUNT; i++) {
	start_child(&childs[i]);
    }

    ev_signal_init(&sigint, exit_on_signal, SIGINT);
    ev_signal_init(&sighup, exit_on_signal, SIGHUP);
    ev_signal_init(&sigterm, exit_on_signal, SIGTERM);

    ev_signal_start(EV_DEFAULT_ &sigint);
    ev_signal_start(EV_DEFAULT_ &sighup);
    ev_signal_start(EV_DEFAULT_ &sigterm);

    ev_loop(EV_DEFAULT_ 0);

    return 0;
}
